"""
atpthings.dict
--------------
Utilities for manipulating with dict (dictionary)


"""
from .get import get_keys, get_values, get_keys_as_list, get_values_from_dict_list
from .io import load, save
from .convert import list_to_dict
